from .__about__ import (
    __author__,
    __commit__,
    __copyright__,
    __email__,
    __license__,
    __summary__,
    __title__,
    __uri__,
    __version__,
)

from . import datasets
from . import make
from . import tarprep
from . import urlcopy

from .fetchfuncs import fetch
